﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Linq;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.DataPreprocessing.Transformations;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;
using HeuristicLab.Parameters;

namespace HeuristicLab.DataPreprocessing.Implementations.Transformations {
  [Item("LinearTransformation", "Represents a linear transformation with multiplication and addition.")]
  public class LinearTransformation : Transformation {

    #region Parameters
    public IValueParameter<DoubleValue> MultiplicandParameter {
      get { return (IValueParameter<DoubleValue>)Parameters["Multiplicand"]; }
    }
    public IValueParameter<DoubleValue> SummandParameter {
      get { return (IValueParameter<DoubleValue>)Parameters["Summand"]; }
    }
    #endregion

    public LinearTransformation()
      : base() {
      Parameters.Add(new ValueParameter<DoubleValue>("Multiplicand", "Multiplicand for linear transformation", new DoubleValue(1.0)));
      Parameters.Add(new ValueParameter<DoubleValue>("Summand", "Summand for linear transformation", new DoubleValue(0.0)));
    }

    public override void Transform() {
      PreprocessingData.InTransaction(() => {
        int idx = PreprocessingData.GetColumnIndex(Column);
        if (PreprocessingData.IsType<double>(idx)) {
          var column = PreprocessingData.GetValues<double>(idx);
          var transformedColumn = column.Select(e => e * MultiplicandParameter.Value.Value + SummandParameter.Value.Value).ToList();
          PreprocessingData.SetValues<double>(idx, transformedColumn);
        }
      },
      DataPreprocessingChangedEventType.Transformation);
    }

    public override ISymbolicExpressionTree GenerateInverseTransformation() {
      throw new System.NotImplementedException();
    }

    public override void Check() {
      throw new System.NotImplementedException();
    }

    public override Common.IDeepCloneable Clone(Common.Cloner cloner) {
      throw new System.NotImplementedException();
    }
  }
}
